MODULE asmbal
   !!======================================================================
   !!                       ***  MODULE asmbal  ***
   !! Assimilation balancing interface: Write to file the balancing increments
   !!                                   calculated for biogeochemistry
   !!======================================================================
   !!----------------------------------------------------------------------
   !!   'key_asminc' : Switch on the assimilation increment interface
   !!----------------------------------------------------------------------
   !!   asm_bal_wri  : Write out the background state
   !!----------------------------------------------------------------------
   !! * Modules used
   USE par_kind, ONLY: &   ! Precision variables
      & wp
   USE iom                 ! I/O module
   USE asminc              ! Main assimilation increments module
   USE asmpar              ! Parameters for the assimilation interface

   IMPLICIT NONE

   !! * Routine accessibility
   PRIVATE
   PUBLIC asm_bal_wri   !: Write out the background state

CONTAINS

   SUBROUTINE asm_bal_wri( kt )
      !!-----------------------------------------------------------------------
      !!
      !!                  ***  ROUTINE asm_bal_wri ***
      !!
      !! ** Purpose : Write to file the balancing increments
      !!              calculated for biogeochemistry
      !!
      !! ** Method  : Write to file the balancing increments
      !!              calculated for biogeochemistry
      !!
      !! ** Action  :
      !!                   
      !! References :
      !!
      !! History    :
      !!        ! 2014-08 (D. Ford)  Initial version, based on asm_bkg_wri
      !!-----------------------------------------------------------------------
      !! * Arguments
      INTEGER, INTENT( IN ) :: kt        ! Current time-step
      !! * Local declarations
      CHARACTER(LEN=50) :: cl_asmbal     ! Filename (with extension)
      LOGICAL           :: llok          ! Check if file exists
      INTEGER           :: inum          ! File unit number
      REAL(wp)          :: zdate         ! Date
      !!-----------------------------------------------------------------------
     
      ! Set things up
      zdate = REAL( ndastp )
      WRITE(cl_asmbal, FMT='(A,".nc")' ) TRIM( c_asmbal )

      ! Check if file exists
      INQUIRE( FILE = TRIM( cl_asmbal ), EXIST = llok )
      IF( .NOT. llok ) THEN
         IF(lwp) WRITE(numout,*) ' Setting up assimilation balancing increments file '// &
            &                    TRIM( c_asmbal ) // ' at timestep = ', kt

         ! Define the output file       
         CALL iom_open( c_asmbal, inum, ldwrt = .TRUE., kiolib = jprstlib)

         ! Write the information
         CALL iom_rstput( kt, kt, inum, 'rdastp' , zdate   )

         IF ( ln_logchltotinc .OR. ln_logchlpftinc ) THEN
#if defined key_fabm
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_chl1', logchl_balinc_ersem_chl1 )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_chl2', logchl_balinc_ersem_chl2 )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_chl3', logchl_balinc_ersem_chl3 )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_chl4', logchl_balinc_ersem_chl4 )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_p1c',  logchl_balinc_ersem_p1c  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_p1n',  logchl_balinc_ersem_p1n  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_p1p',  logchl_balinc_ersem_p1p  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_p1s',  logchl_balinc_ersem_p1s  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_p2c',  logchl_balinc_ersem_p2c  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_p2n',  logchl_balinc_ersem_p2n  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_p2p',  logchl_balinc_ersem_p2p  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_p3c',  logchl_balinc_ersem_p3c  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_p3n',  logchl_balinc_ersem_p3n  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_p3p',  logchl_balinc_ersem_p3p  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_p4c',  logchl_balinc_ersem_p4c  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_p4n',  logchl_balinc_ersem_p4n  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_p4p',  logchl_balinc_ersem_p4p  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_z4c',  logchl_balinc_ersem_z4c  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_z5c',  logchl_balinc_ersem_z5c  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_z5n',  logchl_balinc_ersem_z5n  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_z5p',  logchl_balinc_ersem_z5p  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_z6c',  logchl_balinc_ersem_z6c  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_z6n',  logchl_balinc_ersem_z6n  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_z6p',  logchl_balinc_ersem_z6p  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_n1p',  logchl_balinc_ersem_n1p  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_n3n',  logchl_balinc_ersem_n3n  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_n4n',  logchl_balinc_ersem_n4n  )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_ersem_n5s',  logchl_balinc_ersem_n5s  )
#elif defined key_medusa && defined key_foam_medusa
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_chn', logchl_balinc_medusa_chn )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_chd', logchl_balinc_medusa_chd )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_phn', logchl_balinc_medusa_phn )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_phd', logchl_balinc_medusa_phd )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_pds', logchl_balinc_medusa_pds )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_zmi', logchl_balinc_medusa_zmi )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_zme', logchl_balinc_medusa_zme )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_din', logchl_balinc_medusa_din )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_sil', logchl_balinc_medusa_sil )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_fer', logchl_balinc_medusa_fer )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_det', logchl_balinc_medusa_det )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_dtc', logchl_balinc_medusa_dtc )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_dic', logchl_balinc_medusa_dic )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_alk', logchl_balinc_medusa_alk )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_medusa_oxy', logchl_balinc_medusa_oxy )
#elif defined key_hadocc
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_hadocc_nut', logchl_balinc_hadocc_nut )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_hadocc_phy', logchl_balinc_hadocc_phy )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_hadocc_zoo', logchl_balinc_hadocc_zoo )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_hadocc_det', logchl_balinc_hadocc_det )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_hadocc_dic', logchl_balinc_hadocc_dic )
            CALL iom_rstput( kt, kt, inum, 'logchl_balinc_hadocc_alk', logchl_balinc_hadocc_alk )
#endif
         ENDIF

         CALL iom_close( inum )
      ELSE
         CALL ctl_warn( TRIM( cl_asmbal ) // ' already exists ', &
            &           ' Therefore not writing out balancing increments at this timestep', &
            &           ' Something has probably gone wrong somewhere' )
         IF(lwp) WRITE(numout,*) ' Failed to set up assimilation balancing increments file '// &
            &                    TRIM( c_asmbal ) // ' at timestep = ', kt
      ENDIF
                                 
   END SUBROUTINE asm_bal_wri
END MODULE asmbal
